/*
  Copyright 2010-2011 Grégory Soutadé

  This file is part of KissCount.

  KissCount is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  KissCount is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with KissCount.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "AccountPanel.h"

enum {SEARCH_ID, GRID_ID, CALENDAR_FROM_ID, CALENDAR_TO_ID,
      CHANGE_ACCOUNT_ID, CHANGE_CATEGORY_ID, RENAME_ID};

BEGIN_EVENT_TABLE(SearchPanel, wxPanel)
EVT_BUTTON(SEARCH_ID, SearchPanel::OnButtonSearch)
EVT_CALENDAR_SEL_CHANGED(CALENDAR_FROM_ID, SearchPanel::OnCalendarFromChange)
EVT_CALENDAR_SEL_CHANGED(CALENDAR_TO_ID, SearchPanel::OnCalendarToChange)
EVT_GRID_CMD_CELL_CHANGE(GRID_ID, SearchPanel::OnOperationModified)
EVT_BUTTON(CHANGE_ACCOUNT_ID, SearchPanel::OnButtonChangeAccount)
EVT_BUTTON(CHANGE_CATEGORY_ID, SearchPanel::OnButtonChangeCategory)
EVT_BUTTON(RENAME_ID, SearchPanel::OnButtonRename)
EVT_SHOW(SearchPanel::OnShow)
END_EVENT_TABLE()

#define UNESCAPE_CHARS(s) {                             \
            s.Replace(wxT("\\\""), wxT("\""), true);    \
            s.Replace(wxT("\\\'"), wxT("\'"), true);    \
    }

SearchPanel::SearchPanel(KissCount* kiss, wxUI *parent) : KissPanel(kiss, parent), _operations(NULL)
{
    DEFAULT_FONT(font);
    User* user = _kiss->GetUser();
    std::vector<Account>::iterator accountIt;
    std::vector<Category>::iterator categoryIt;
    wxDateTime firstOfMonth;
    wxRect rect = wxDisplay().GetGeometry();

    wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *vbox2 = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *hbox = new wxBoxSizer(wxHORIZONTAL);

    SetSizer(vbox);

    _checkDateFrom = new wxCheckBox(this, wxID_ANY, _("Date from"));
    _checkDateTo = new wxCheckBox(this, wxID_ANY, _("Date to"));

    _checkDateFrom->SetValue(wxT("1"));
    _checkDateTo->SetValue(wxT("1"));

    wxGridBagSizer *gridBagSizer = new wxGridBagSizer(3, 10);

    firstOfMonth.SetToCurrent();
    firstOfMonth.SetDay(1);
    _calendarFrom = new wxCalendarCtrl(this, CALENDAR_FROM_ID, firstOfMonth, wxDefaultPosition, wxDefaultSize, 
                                       wxCAL_MONDAY_FIRST);
    _calendarTo = new wxCalendarCtrl(this, CALENDAR_TO_ID, wxDefaultDateTime, wxDefaultPosition, wxDefaultSize, 
                                     wxCAL_MONDAY_FIRST);

  
    _description = new wxTextCtrl(this, wxID_ANY);
    wxSize size = _description->GetSize();
    size.SetWidth(size.GetWidth()*2);
    _description->SetMinSize(size);
    _amountFrom = new wxTextCtrl(this, wxID_ANY);
    _amountTo = new wxTextCtrl(this, wxID_ANY);

    _category = new wxCheckListBox(this, wxID_ANY);
    _category->Append(_("Unknown"));
    for(categoryIt = user->_categories.begin(); categoryIt != user->_categories.end(); categoryIt++)
        _category->Append(wxGetTranslation(categoryIt->name));

    wxString stypes[] = {_("Fix"), _("Non fix"), _("Checked"), _("Not checked")};
    _optype = new wxCheckListBox(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, 4, stypes);

    _account = new wxCheckListBox(this, wxID_ANY);
    _account->Append(_("Unknown"));
    for(accountIt = user->_accounts.begin(); accountIt != user->_accounts.end(); accountIt++)
        _account->Append(accountIt->name);

    _searchButton = new wxButton(this, SEARCH_ID, _("Search"));

    wxStaticText* labelDescription = new wxStaticText(this, wxID_ANY, _("Description"));
    wxStaticText* labelAmountFrom = new wxStaticText(this, wxID_ANY, _("Amount from"));
    wxStaticText* labelAmountTo = new wxStaticText(this, wxID_ANY, _("Amount to"));
    wxStaticText* labelCategory = new wxStaticText(this, wxID_ANY, _("Category"));
    wxStaticText* labelOperations = new wxStaticText(this, wxID_ANY, _("Operations"));
    wxStaticText* labelAccount = new wxStaticText(this, wxID_ANY, _("Account"));

    gridBagSizer->Add(labelDescription, wxGBPosition(0, 0));
    gridBagSizer->Add(_description, wxGBPosition(1, 0));
    gridBagSizer->Add(_checkDateFrom, wxGBPosition(0, 1));
    gridBagSizer->Add(_calendarFrom, wxGBPosition(1, 1));
    gridBagSizer->Add(_checkDateTo, wxGBPosition(0, 2));
    gridBagSizer->Add(_calendarTo, wxGBPosition(1, 2));
    gridBagSizer->Add(labelAmountFrom, wxGBPosition(0, 3));
    gridBagSizer->Add(_amountFrom, wxGBPosition(1, 3));
    gridBagSizer->Add(labelAmountTo, wxGBPosition(0, 4));
    gridBagSizer->Add(_amountTo, wxGBPosition(1, 4));
    gridBagSizer->Add(labelCategory, wxGBPosition(0, 5));
    gridBagSizer->Add(_category, wxGBPosition(1, 5));
    gridBagSizer->Add(labelOperations, wxGBPosition(0, 6));
    gridBagSizer->Add(_optype, wxGBPosition(1, 6));
    gridBagSizer->Add(labelAccount, wxGBPosition(0, 7));
    gridBagSizer->Add(_account, wxGBPosition(1, 7));
    gridBagSizer->Add(_searchButton, wxGBPosition(2, 0));

    vbox->Add(gridBagSizer, 0, wxGROW|wxALL, 5);

    _grid = new GridAccount(_kiss, this, GRID_ID);

    hbox->Add(_grid, 0, wxGROW|wxALL, 5);

    _changeAccountButton = new wxButton(this, CHANGE_ACCOUNT_ID, _("Change account"));
    _changeCategoryButton = new wxButton(this, CHANGE_CATEGORY_ID, _("Change category"));
    _renameButton = new wxButton(this, RENAME_ID, _("Rename"));

    vbox2->Add(_changeAccountButton, wxALL, 15);
    vbox2->Add(_changeCategoryButton, wxALL, 15);
    vbox2->Add(_renameButton, wxALL, 15);

    hbox->Add(vbox2, 0, wxALL, 15);

    vbox->Add(hbox, 0, wxGROW|wxALL, 5);

    Fit();

    SetMinSize(wxSize(rect.width-rect.x-15, rect.height-rect.y-128-25));
    SetMaxSize(wxSize(rect.width-rect.x-15, rect.height-rect.y-128-25));
    SetScrollbars(10, 10, 100/10, 100/10);
}

SearchPanel::~SearchPanel()
{
    if (_operations) delete _operations;
}

KissPanel* SearchPanel::CreatePanel()
{
    return new SearchPanel(_kiss, _wxUI);
}

wxBitmapButton* SearchPanel::GetButton(int id)
{
    if (!_KissButton)
	_KissButton = new wxBitmapButton(_wxUI, id, wxBitmap(wxT(SEARCH_ICON), wxBITMAP_TYPE_PNG), wxDefaultPosition, wxSize(128, 128));

    return _KissButton;
}

wxString SearchPanel::GetToolTip()
{
    return _("Search");
}

void SearchPanel::Search()
{
    wxString *description=NULL, *amountFrom=NULL, *amountTo=NULL;
    std::vector<wxString> categories, accounts;
    wxDateTime *dateFrom=NULL, *dateTo=NULL;
    User* user= _kiss->GetUser();
    int i, types=0;
    std::vector<Operation>::iterator it;
    double af, at;

    if (_calendarFrom->GetDate() > _calendarTo->GetDate())
    {
        wxMessageBox(_("Invalid date range"), _("Error"), wxICON_ERROR | wxOK);      
        return;
    }

    if (_amountFrom->GetLineText(0).Length())
    {
        amountFrom = new wxString;
        *amountFrom = _amountFrom->GetLineText(0);
        if (!amountFrom->ToDouble(&af))
	{
            wxMessageBox(_("Invalid amount from"), _("Error"), wxICON_ERROR | wxOK);      
            delete amountFrom;
            return;
	}

        if (af < 0) af *= -1;
    }

    if (_amountTo->GetLineText(0).Length())
    {
        amountTo = new wxString;
        *amountTo = _amountTo->GetLineText(0);
        if (!amountTo->ToDouble(&at))
	{
            wxMessageBox(_("Invalid amount to"), _("Error"), wxICON_ERROR | wxOK);
            delete amountFrom;
            delete amountTo;
            return;
	}

        if (at < 0) at *= -1;
    }

    if (amountFrom && amountTo && af > at)
    {
        wxMessageBox(_("Invalid amount range"), _("Error"), wxICON_ERROR | wxOK);
        delete amountFrom;
        delete amountTo;
        return;
    }

    _grid->DeleteRows(1, _grid->GetNumberRows()-1);

    if (_description->GetLineText(0).Length())
    {
        description = new wxString;
        *description = _description->GetLineText(0);
    }

    if (_checkDateFrom->IsChecked())
    {
        dateFrom = new wxDateTime;
        *dateFrom = _calendarFrom->GetDate();
    }

    if (_checkDateTo->IsChecked())
    {
        dateTo = new wxDateTime;
        *dateTo = _calendarTo->GetDate();
    }

    if (dateFrom && dateTo && *dateFrom > *dateTo)
    {
        ;
    }

    for(i=0; i<user->GetCategoriesNumber(); i++)
        if (_category->IsChecked(i))
            categories.push_back((i) ? user->_categories[i-1].id : wxT("0"));

    types |= (_optype->IsChecked(0)) ? FIX_OP         : 0;
    types |= (_optype->IsChecked(1)) ? NON_FIX_OP     : 0;
    types |= (_optype->IsChecked(2)) ? CHECKED_OP     : 0;
    types |= (_optype->IsChecked(3)) ? NOT_CHECKED_OP : 0;

    for(i=0; i<user->GetAccountsNumber(); i++)
        if (_account->IsChecked(i))
            accounts.push_back((i) ? user->_accounts[i-1].id : wxT("0"));

    if (_operations) 
        delete _operations;

    _operations = _kiss->Search(description, dateFrom, dateTo, amountFrom, amountTo, categories,types, accounts);
}

void SearchPanel::OnButtonSearch(wxCommandEvent& event)
{
    Search();

    if (_operations->size() > 1)
        wxMessageBox(wxString::Format(wxT("%d"), _operations->size()) + _(" entries found"), wxT("KissCount"), wxICON_INFORMATION | wxOK);
    else if (_operations->size() == 1)
        wxMessageBox(_("1 entry found"), wxT("KissCount"), wxICON_INFORMATION | wxOK);
    else
    {
        wxMessageBox(_("No entry found"), wxT("KissCount"), wxICON_INFORMATION | wxOK);
        return;
    }

    _grid->LoadOperations(_operations, false, false, 0, 0);

    _wxUI->Layout();
}

static void ChangeAccount(Operation* op, void** params)
{
    wxString* account = (wxString*) params[0];

    op->account = *account;
}

void SearchPanel::OnButtonChangeAccount(wxCommandEvent& event)
{
    int i, a;
    std::vector<int> rows;
    User* user = _kiss->GetUser();
    wxString *accounts = new wxString[user->GetAccountsNumber()+1];
    std::vector<Operation>::iterator it;
    wxString account;
    void * params[] = {&account};

    if (!_operations) return;

    _grid->GetSelectedOperations(&rows);

    accounts[0] = _("None");
    a = 0;
    for(i=0; i < user->GetAccountsNumber(); i++)
	accounts[++a] = user->_accounts[i].name;

    wxSingleChoiceDialog dialog(_wxUI, _("Choose a new account"), wxT("KissCount"), user->GetAccountsNumber()+1, accounts);

    if (dialog.ShowModal() == wxID_CANCEL)
	return;

    a = dialog.GetSelection();
    account =  (a) ? user->_accounts[a-1].id : wxT("0");

    _grid->MassUpdate(rows, ChangeAccount, params);

    _wxUI->NeedReload();
}

static void ChangeCategory(Operation* op, void** params)
{
    wxString* category = (wxString*) params[0];
    bool* fix = (bool*) params[1];

    op->category = *category;
    op->fix_cost = * fix;
}

void SearchPanel::OnButtonChangeCategory(wxCommandEvent& event)
{
    int i, a;
    std::vector<int> rows;
    User* user = _kiss->GetUser();
    wxString *categories = new wxString[user->GetCategoriesNumber()+1];
    std::vector<Operation>::iterator it;
    wxString category;
    bool fix;
    void * params[] = {&category, &fix};

    if (!_operations) return;

    _grid->GetSelectedOperations(&rows);

    categories[0] = _("None");
    a = 0;
    for(i=0; i < user->GetCategoriesNumber(); i++)
	categories[++a] = user->_categories[i].name;

    wxSingleChoiceDialog dialog(_wxUI, _("Choose a new category"), wxT("KissCount"), user->GetCategoriesNumber()+1, categories);

    if (dialog.ShowModal() == wxID_CANCEL)
	return;

    a = dialog.GetSelection();
    
    if (a)
    {
	category = user->_categories[a-1].id ;
	fix = user->_categories[a-1].fix_cost;
    }
    else
    {
	category = wxT("0");
	fix = false;
    }

    _grid->MassUpdate(rows, ChangeCategory, params);

    _wxUI->NeedReload();
}

static void ChangeName(Operation* op, void** params)
{
    wxString* description = (wxString*) params[0];

    op->description = *description;
}

void SearchPanel::OnButtonRename(wxCommandEvent& event)
{
    std::vector<int> rows;
    std::vector<Operation>::iterator it;
    wxString category;
    wxString description;
    void * params[] = {&description};

    if (!_operations) return;

    _grid->GetSelectedOperations(&rows);

    wxTextEntryDialog u(this, wxT(""), _("Enter a new description"));
  
    if (u.ShowModal() == wxID_CANCEL)
        return;

    description = u.GetValue();

    if (!description.size()) return;

    _grid->MassUpdate(rows, ChangeName, params);

    _wxUI->NeedReload();
}

void SearchPanel::OnShow(wxShowEvent& event)
{
    _wxUI->SetTitle(_kiss->GetUser()->_name + wxT(" - ") + _("Search"));
}

void SearchPanel::OnCalendarFromChange(wxCalendarEvent& event)
{
    _checkDateFrom->SetValue(true);
}

void SearchPanel::OnCalendarToChange(wxCalendarEvent& event)
{
    _checkDateTo->SetValue(true);
}

void SearchPanel::OnOperationModified(wxGridEvent& event)
{
    _wxUI->NeedReload();
}
