/*
  Copyright 2010-2011 Grégory Soutadé

  This file is part of KissCount.

  KissCount is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  KissCount is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with KissCount.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef GRIDACCOUNT_H
#define GRIDACCOUNT_H

#include <wx/wx.h>
#include <wx/grid.h>
#include <list>
#include <vector>
#include <algorithm>
#include <view/AccountPanel.h>
#include <model/model.h>
#include <controller/KissCount.h>
#include "wxGridCellFastBoolEditor.h"
#include "wxGridCellTreeButtonRenderer.h"
#include "wxGridCellTreeButtonEditor.h"
#include "wxGridCellFormulaEditor.h"
#include "wxGridCellTabStringRenderer.h"

class KissCount;

typedef void (*updateOperationFunc)(Operation* op, void** params);

class GridAccount : public wxGrid
{
public:
  GridAccount(KissCount* kiss, wxWindow *parent, wxWindowID id);
    ~GridAccount();
  
    wxPen GetColGridLinePen (int col);
    wxPen GetRowGridLinePen (int row);

    void LoadOperations(std::vector<Operation>* operations, bool canAddOperation, bool setWeek, int month, int year);
    void InsertOperationWithWeek(User* user, Operation& op, int line, bool fix, int month, int year) ;
    void InsertOperation(User* user, Operation& op, int line, bool fix, int month, int year) ;

    void GetSelectedOperations(std::vector<int>* rows);

    void MassUpdate(std::vector<int>& rows, updateOperationFunc func, void** params);

    void Group();
    void UnGroup();

    void OnCellLeftClick(wxGridEvent& evt);
    void OnOperationModified(wxGridEvent& event);

    int _fixCosts;
    int _week1, _week2, _week3, _week4;
    std::vector<Operation> _displayedOperations;

private:
    wxWindow* _parent;
    KissCount* _kiss;
    bool _displayLines;
    wxString* _categories, *_accounts;
    std::vector<Operation>* _operations;
    bool _canAddOperation, _loadOperations;
    int _curMonth, _curYear;

    void SetWeek(int week, int line);
    void ResetWeeks();
    void ComputeWeeks();

    void InsertIntoGrid(Operation& op);
    void DeleteOperation(const Operation& op);
    void UpdateMeta(Operation& op);
    int  RemoveMeta(Operation op, int line, bool removeRoot, bool deleteOp);
    void CheckMeta(Operation& op, int line, bool check);

    Operation& GetOperation(const wxString& id);
    void UpdateOperation(Operation& op);
    int GetDisplayedRow(const wxString& id);

    DECLARE_EVENT_TABLE();
};
#endif
